#!/usr/bin/perl

# The LearningOnline Network with CAPA
# Converts a texxml file into a single tex file
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# 7-16-2002 Jeremy Bowers

use strict;
use HTML::TokeParser;
use GDBM_File;

# accept texxml document on standard in
my $p = HTML::TokeParser->new( $ARGV[0] );
my $dirprefix = "/home/httpd/html/adm/help/tex/";

# Print the header
open (LATEX_FILE, $dirprefix . "Latex_Header.tex");
print <LATEX_FILE>;


while (my $token = $p->get_token())
{
    my $type = $token->[0];
    if ($type eq 'S')
    {
	my $tag = $token->[1];
	my $attr = $token->[2];
	if ($tag eq 'section')
	{
	    my $title = $attr->{'name'};
	    print "\\section{$title}\n\n";
	}

	if ($tag eq 'subsection')
	{
	    my $title = $attr->{'name'};
	    print "\\subsection{$title}\n\n";
	}

	if ($tag eq 'subsubsection')
	{
	    my $title = $attr->{'name'};
	    print "\\subsubsection{$title}\n\n";
	}

	if ($tag eq 'file')
	{
	    my $file = $attr->{'name'};
	    open (LATEX_FILE, $dirprefix . $file);
	    print <LATEX_FILE>;
	    print "\n\n";
	}

	if ($tag eq 'tex')
	{
	    print "\n\n";
	    print $attr->{'content'};
	    print "\n\n";
	}
    }
}

# Print out the footer.
open (LATEX_FILE, $dirprefix . "Latex_Footer.tex");
print <LATEX_FILE>;
