# The LearningOnline Network with CAPA
# Quick Student Grades Display
#
# $Id:
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# Created Nov. 14, 2002 by Jeremy Bowers

package Apache::lonquickgrades;

use strict;
use Apache::Constants qw(:common :http);
use POSIX;

sub handler {
    my $r = shift;
    return real_handler($r);
}

sub real_handler {
    my $r = shift;

    &Apache::loncommon::get_unprocessed_cgi($ENV{QUERY_STRING});

    # Handle header-only request
    if ($r->header_only) {
        if ($ENV{'browser.mathml'}) {
            $r->content_type('text/xml');
        } else {
            $r->content_type('text/html');
        }
        $r->send_http_header;
        return OK;
    }

    # Send header, don't cache this page
    if ($ENV{'browser.mathml'}) {
        $r->content_type('text/xml');
    } else {
        $r->content_type('text/html');
    }
    &Apache::loncommon::no_cache($r);
    $r->send_http_header;

    # Create the nav map
    my $navmap = Apache::lonnavmaps::navmap->new(
                        $ENV{"request.course.fn"}.".db",
                        $ENV{"request.course.fn"}."_parms.db", 1, 0);

    if (!defined($navmap)) {
        my $requrl = $r->uri;
        $ENV{'user.error.msg'} = "$requrl:bre:0:0:Navamp initialization failed.";
        return HTTP_NOT_ACCEPTABLE;
    }

    # Keep this hash in sync with %statusIconMap in lonnavmaps; they
    # should match color/icon
    my $res = $navmap->firstResource(); # temp resource to access constants
 
    # Header
    $r->print(&Apache::loncommon::bodytag('Quick Score Display','',
                                          ''));

    $r->print(<<HEADER);
<p>This screen shows how many problems (or problem parts) you have completed, and
how many you have not yet done. You can also look at <a href="/adm/studentcalc">a detailed
score sheet</a>.</p>
HEADER

    $navmap->init();

    # End navmap using boilerplate

    my $iterator = $navmap->getIterator(undef, undef, undef, 1);
    my $depth = 1;
    $iterator->next(); # ignore first BEGIN_MAP
    my $curRes = $iterator->next();
    
    # General overview of the following: Walk along the course resources.
    # For every problem in the resource, tell its parent maps how many
    # parts and how many parts correct it has. After that, each map will
    # have a count of the total parts underneath it, correct and otherwise.
    # After that, we will walk through the course again and read off
    # maps in order, with their data. 
    # (If in the future people decide not to be cumulative, only add
    #  the counts to the parent map.)

    my $totalParts = 0; my $totalPossible = 0; my $totalRight = 0;

    # Pre-run: Count parts correct
    while ( $depth > 0 ) {
        if ($curRes == $iterator->BEGIN_MAP()) {$depth++;}
        if ($curRes == $iterator->END_MAP()) { $depth--; }

        if (ref($curRes) && $curRes->is_problem() && !$curRes->randomout)
        {
            # Get number of correct, incorrect parts
            my $parts = $curRes->parts();
            if (scalar(@{$parts}) > 1) { shift @{$parts}; } # lose the first "0"
            $totalParts += scalar(@{$parts});
            my $partsRight = 0;
            my $stack = $iterator->getStack();
            
            for my $part (@{$parts}) {
                my $status = $curRes->getCompletionStatus($part);
                if ($status == $curRes->CORRECT || $status == $curRes->CORRECT_BY_OVERRIDE
                    || $status == $curRes->EXCUSED) {
                    $partsRight++;
                    $totalRight++;
                }

                my $dateStatus = $curRes->getDateStatus($part);
                if ($status != $curRes->OPEN_LATER) {
                    $totalPossible++;
                }
            }
            
            # Crawl down stack and record parts correct and total
            for my $res (@{$stack}) {
                if (ref($res) && $res->is_map()) {
                    if (!defined($res->{DATA}->{CHILD_PARTS})) {
                        $res->{DATA}->{CHILD_PARTS} = 0;
                        $res->{DATA}->{CHILD_CORRECT} = 0;
                    }
                    
                    $res->{DATA}->{CHILD_PARTS} += scalar(@{$parts});
                    $res->{DATA}->{CHILD_CORRECT} += $partsRight;
                }
            }
        }
        $curRes = $iterator->next();
    }

    $iterator = $navmap->getIterator(undef, undef, undef, 1);
    $depth = 1;
    $iterator->next(); # ignore first BEGIN_MAP
    $curRes = $iterator->next();

    my @start = (255, 255, 192);
    my @end   = (0, 192, 0);

    my $indentString = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';

    my $unaccountedTotal = $totalParts;
    my $unaccountedCorrect = $totalRight;

    # Second pass: Print the maps.
    $r->print('<table cellspacing="3" border="2"><tr><td align="center"><b>Sequence</b></td>');
    $r->print('<td align="center">Done / Total</td><tr>' . "\n\n");
    while ($depth > 0) {
        if ($curRes == $iterator->BEGIN_MAP()) {$depth++;}
        if ($curRes == $iterator->END_MAP()) { $depth--; }

        if (ref($curRes) && $curRes->is_map()) {
            my $stack = $iterator->getStack();
            my $src = Apache::lonnavmaps::getLinkForResource($stack);
            my $srcHasQuestion = $src =~ /\?/;
            my $link = $src.
                ($srcHasQuestion?'&':'?') .
                'symb='.&Apache::lonnet::escape($curRes->symb()).
                '"';
            my $title = $curRes->compTitle();
            
            my $correct = $curRes->{DATA}->{CHILD_CORRECT};
            my $total = $curRes->{DATA}->{CHILD_PARTS};

            $unaccountedTotal -= $total;
            $unaccountedCorrect -= $correct;

            if ($total > 0) {
                my $ratio;
                $ratio = $correct / $total;
                my $color = mixColors(\@start, \@end, $ratio);
                $r->print("<tr><td bgcolor='$color'>");
                
                for (my $i = 1; $i < $depth; $i++) { $r->print($indentString); }
                
                $r->print("<a href='$link'>$title</a></td>");
                $r->print("<td align='center'>$correct / $total</td></tr>\n");
            }
        }

        $curRes = $iterator->next();
    }

    # If there were any problems at the top level, print an extra "catchall"
    if ($unaccountedTotal > 0) {
        my $ratio = $unaccountedCorrect / $unaccountedTotal;
        my $color = mixColors(\@start, \@end, $ratio);
        $r->print("<tr><td bgcolor='$color'>");
        $r->print("Problems Not Contained In Sequences</td><td align='center'>");
        $r->print("$unaccountedCorrect / $unaccountedTotal</td></tr>");
    }

    my $maxHelpLink = Apache::loncommon::help_open_topic("Quick_Grades_Possibly_Correct");

    $r->print("<tr><td colspan='2' align='right'>Total Parts Done: <b>$totalRight</b><br>");
    $r->print("Max Possible To Date $maxHelpLink: <b>$totalPossible</b><br>");
    $r->print("Total Parts In Course: <b>$totalParts</b></td></tr>\n\n");


    $r->print("</table></body></html>");

    $navmap->untieHashes();

    return OK;
}

# Pass this two refs to arrays for the start and end color, and a number
# from 0 to 1 for how much of the latter you want to mix in. It will
# return a string ready to show ("#FFC309");
sub mixColors {
    my $start = shift;
    my $end = shift;
    my $ratio = shift;
    
    my ($a,$b);
    my $final = "";
    $a = $start->[0]; $b = $end->[0];
    my $mix1 = POSIX::floor((1-$ratio)*$a + $ratio*$b);
    $a = $start->[1]; $b = $end->[1];
    my $mix2 = POSIX::floor((1-$ratio)*$a + $ratio*$b);
    $a = $start->[2]; $b = $end->[2];
    my $mix3 = POSIX::floor((1-$ratio)*$a + $ratio*$b);

    $final = sprintf "%2x%2x%2x", $mix1, $mix2, $mix3;
    return "#" . $final;
}

1;
