# The LearningOnline Network with CAPA
#
# $Id: lonstudentassessment.pm,v 1.48 2003/05/21 14:43:01 matthew Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# (Navigate problems for statistical reports
#
#######################################################
#######################################################

=pod

=head1 NAME

lonstudentassessment

=head1 SYNOPSIS

Presents assessment data about a student or a group of students.

=head1 Subroutines

=over 4 

=cut

#######################################################
#######################################################

package Apache::lonstudentassessment;

use strict;
use Apache::lonstatistics;
use Apache::lonhtmlcommon;
use Apache::loncoursedata;
use Apache::lonnet; # for logging porpoises
use Spreadsheet::WriteExcel;

#######################################################
#######################################################
=pod

=item Package Variables

=over 4

=item $Statistics Hash ref to store student data.  Indexed by symb,
      contains hashes with keys 'score' and 'max'.

=cut

#######################################################
#######################################################

my $Statistics;

#######################################################
#######################################################

=pod

=item $show_links 'yes' or 'no' for linking to student performance data

=item $output_mode 'html', 'excel', or 'csv' for output mode

=item $show 'all', 'totals', or 'scores' determines how much data is output

=cut

#######################################################
#######################################################
my $show_links;
my $output_mode;
my $show;

#######################################################
#######################################################
# End of package variable declarations

=pod

=back

=cut

#######################################################
#######################################################

=pod

=item &BuildStudentAssessmentPage()

Inputs: 

=over 4

=item $r Apache Request

=item $c Apache Connection 

=back

=cut

#######################################################
#######################################################
sub BuildStudentAssessmentPage {
    my ($r,$c)=@_;
    undef($Statistics);
    #
    # Print out the HTML headers for the interface
    #    This also parses the output mode selector
    #    This step must always be done.
    $r->print(&CreateInterface());
    $r->print('<input type="hidden" name="notfirstrun" value="true" />');
    $r->rflush();
    if (! exists($ENV{'form.notfirstrun'})) {
        $r->print(<<ENDMSG);
<p>
<font size="+2">
Please make your selections in the boxes above and hit 
the button marked &quot;Update&nbsp;Display&quot;.
</font>
</p>
ENDMSG
#        $r->print(&OutputDescriptions());
        return;
    }
    #
    #
    my $initialize     = \&html_initialize;
    my $output_student = \&html_outputstudent;
    my $finish         = \&html_finish;
    #
    if ($output_mode eq 'excel') {
        $initialize     = \&excel_initialize;
        $output_student = \&excel_outputstudent;
        $finish         = \&excel_finish;
#    } elsif ($output_mode eq 'multi-sheet excel') {
#        $initialize     = \&multi_sheet_excel_initialize;
#        $output_student = \&multi_sheet_excel_outputstudent;
#        $finish         = \&multi_sheet_excel_finish;
    } elsif ($output_mode eq 'csv') {
        $initialize     = \&csv_initialize;
        $output_student = \&csv_outputstudent;
        $finish         = \&csv_finish;
    }
    #
    if($c->aborted()) {  return ; }
    #
    # Call the initialize routine selected above
    $initialize->($r);
    foreach my $student (@Apache::lonstatistics::Students) {
        if($c->aborted()) { 
            $finish->($r);
            return ; 
        }
        # Call the output_student routine selected above
        $output_student->($r,$student);
    }
    # Call the "finish" routine selected above
    $finish->($r);
    #
    return;
}

#######################################################
#######################################################

sub get_student_fields_to_show {
    my @to_show = @Apache::lonstatistics::SelectedStudentData;
    foreach (@to_show) {
        if ($_ eq 'all') {
            @to_show = @Apache::lonstatistics::StudentDataOrder;
            last;
        }
    }
    return @to_show;
}

#######################################################
#######################################################

=pod

=item &CreateInterface()

Called by &BuildStudentAssessmentPage to create the top part of the
page which displays the chart.

Inputs: None

Returns:  A string containing the HTML for the headers and top table for 
the chart page.

=cut

#######################################################
#######################################################
sub CreateInterface {
    my $Str = '';
#    $Str .= &CreateLegend();
    $Str .= '<table cellspacing="5">'."\n";
    $Str .= '<tr>';
    $Str .= '<td align="center"><b>Sections</b></td>';
    $Str .= '<td align="center"><b>Student Data</b></td>';
    $Str .= '<td align="center"><b>Enrollment Status</b></td>';
    $Str .= '<td align="center"><b>Sequences and Folders</b></td>';
    $Str .= '<td align="center"><b>Output Format</b></td>';
    $Str .= '</tr>'."\n";
    #
    $Str .= '<tr><td align="center">'."\n";
    $Str .= &Apache::lonstatistics::SectionSelect('Section','multiple',5);
    $Str .= '</td><td align="center">';
    my $only_seq_with_assessments = sub { 
        my $s=shift;
        if ($s->{'num_assess'} < 1) { 
            return 0;
        } else { 
            return 1;
        }
    };
    $Str .= &Apache::lonstatistics::StudentDataSelect('StudentData','multiple',
                                                      5,undef);
    $Str .= '</td><td>'."\n";
    $Str .= &Apache::lonhtmlcommon::StatusOptions(undef,undef,5);
    $Str .= '</td><td>'."\n";
    $Str .= &Apache::lonstatistics::MapSelect('Maps','multiple,all',5,
                                              $only_seq_with_assessments);
    $Str .= '</td><td>'."\n";
    $Str .= &CreateAndParseOutputSelector();
    $Str .= '</td></tr>'."\n";
    $Str .= '</table>'."\n";
    return $Str;
}

#######################################################
#######################################################

=pod

=item &CreateAndParseOutputSelector()

=cut

#######################################################
#######################################################
my @OutputOptions = 
    ({ name  => 'HTML, with links',
       value => 'html, with links',
       description => 'Output HTML with each symbol linked to the problem '.
	   'which generated it.',
       mode => 'html',
       show => 'all',
       show_links => 'yes',
       },
     { name  => 'HTML, with all links',
       value => 'html, with all links',
       description => 'Output HTML with each symbol linked to the problem '.
	   'which generated it.  '.
           'This includes links for unattempted problems.',
       mode => 'html',
       show => 'all',
       show_links => 'all',
       },
     { name  => 'HTML, without links',
       value => 'html, without links',
       description => 'Output HTML.  By not including links, the size of the'.
	   ' web page is greatly reduced.  If your browser crashes on the '.
	   'full display, try this.',
       mode => 'html',
       show => 'all',
       show_links => 'no',
           },
     { name  => 'HTML, scores only',
       value => 'html, scores only',
       description => 'Output HTML, only showing the total number of correct'.
	   ' problems (or problem parts) and not the maximum possible for '.
	   'each student',
       mode => 'html',
       show => 'scores',
       show_links => 'no',
           },
     { name  => 'HTML, totals',
       value => 'html, totals',
       description => 'Output HTML, but only the summary statistics for each'.
	   ' sequence selected for each student.',
       mode => 'html',
       show => 'totals',
       show_links => 'no',
       },
     { name  => 'HTML, summary table only',
       value => 'html summary table only',
       description => 'Output HTML, but only the final summary table for '.
           'all students across all sequences.',
       mode => 'html',
       show => 'final table',
       show_links => 'no',
       },
     { name  => 'Excel, scores only',
       value => 'excel, scores only',
       description => 'Output an Excel file (compatable with Excel 95), '.
	   'with a single column for each sequence showing the students '.
	   'score.',
       mode => 'excel',
       show => 'scores',
       show_links => 'no',
           },
     { name  => 'Excel, totals',
       value => 'excel, totals',
       description => 'Output an Excel file (compatable with Excel 95), '.
	   'with two columns for each sequence, the students score on the '.
	   'sequence and the students maximum possible on the sequence',
       mode => 'excel',
       show => 'totals',
       show_links => 'no',
           },
#     { name  => 'multi-sheet Excel',
#       value => 'multi-sheet excel',
#       description => 'Output an Excel file (compatable with Excel 95), '.
#	   'with a seperate worksheet for each sequence you have selected '.
#           'the data for each problem part '.
#           '(number of tries, status, points awarded) will be listed.',
#       mode => 'multi-sheet excel',
#       show => 'totals',
#       show_links => 'no',
#           },
#     { name  => 'multi-sheet Excel, by section',
#       value => 'multi-sheet excel, by section',
#       description => 'Output an Excel file (compatable with Excel 95), '.
#	   'with a seperate worksheet for each sequence you have selected '.
#           'the data for each problem part '.
#           '(number of tries, status, points awarded) will be listed.  '.
#           'There will be one Excel workbook for each section selected.',
#       mode => 'multi-sheet excel',
#       show => 'by section',
#       show_links => 'no',
#           },
     { name  => 'CSV, everything',
       value => 'csv, everything',
       description => '',
       mode => 'csv',
       show => 'all',
       show_links => 'no',
           },
     { name  => 'CSV, scores only',
       value => 'csv, scores only',
       description => '',
       mode => 'csv',
       show => 'scores',
       show_links => 'no',
           },
     { name  => 'CSV, totals',
       value => 'csv, totals',
       description => '',
       mode => 'csv',
       show => 'totals',
       show_links => 'no',
           },
     );

sub OutputDescriptions {
    my $Str = '';
    $Str .= "<h2>Output Modes</h2>\n";
    $Str .= "<dl>\n";
    foreach my $outputmode (@OutputOptions) {
	$Str .="    <dt>".$outputmode->{'name'}."</dt>\n";
	$Str .="        <dd>".$outputmode->{'description'}."</dd>\n";
    }
    $Str .= "</dl>\n";
    return $Str;
}

sub CreateAndParseOutputSelector {
    my $Str = '';
    my $elementname = 'chartoutputmode';
    #
    # Format for output options is 'mode, restrictions';
    my $selected = 'html, with links';
    if (exists($ENV{'form.'.$elementname})) {
        if (ref($ENV{'form.'.$elementname} eq 'ARRAY')) {
            $selected = $ENV{'form.'.$elementname}->[0];
        } else {
            $selected = $ENV{'form.'.$elementname};
        }
    }
    #
    # Set package variables describing output mode
    $show_links  = 'no';
    $output_mode = 'html';
    $show        = 'all';
    foreach my $option (@OutputOptions) {
        next if ($option->{'value'} ne $selected);
        $output_mode = $option->{'mode'};
        $show        = $option->{'show'};
        $show_links  = $option->{'show_links'};
    }

    #
    # Build the form element
    $Str = qq/<select size="5" name="$elementname">/;
    foreach my $option (@OutputOptions) {
        $Str .= "\n".'    <option value="'.$option->{'value'}.'"';
        $Str .= " selected " if ($option->{'value'} eq $selected);
        $Str .= ">".$option->{'name'}."<\/option>";
    }
    $Str .= "\n</select>";
    return $Str;
}

#######################################################
#######################################################

=pod

=head2 HTML output routines

=item &html_initialize($r)

Create labels for the columns of student data to show.

=item &html_outputstudent($r,$student)

Return a line of the chart for a student.

=item &html_finish($r)

=cut

#######################################################
#######################################################
{
    my $padding;
    my $count;

    my $nodata_count; # The number of students for which there is no data
    my %prog_state;   # progress state used by loncommon PrgWin routines

sub html_initialize {
    my ($r) = @_;
    #
    $padding = ' 'x3;
    $count = 0;
    $nodata_count = 0;
    #
    $r->print("<h3>".$ENV{'course.'.$ENV{'request.course.id'}.'.description'}.
              "&nbsp;&nbsp;".localtime(time)."</h3>");

    #
    # Set up progress window for 'final table' display only
    if ($show eq 'final table') {
        my $studentcount = scalar(@Apache::lonstatistics::Students); 
        %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
            ($r,'Summary Table Status',
             'Summary Table Compilation Progress', $studentcount);
    }
    my $Str = "<pre>\n";
    # First, the @StudentData fields need to be listed
    my @to_show = &get_student_fields_to_show();
    foreach my $field (@to_show) {
        my $title=$Apache::lonstatistics::StudentData{$field}->{'title'};
        my $base =$Apache::lonstatistics::StudentData{$field}->{'base_width'};
        my $width=$Apache::lonstatistics::StudentData{$field}->{'width'};
        $Str .= $title.' 'x($width-$base).$padding;
    }
    # Now the selected sequences need to be listed
    foreach my $sequence (&Apache::lonstatistics::Sequences_with_Assess()){
        my $title = $sequence->{'title'};
        my $base  = $sequence->{'base_width'};
        my $width = $sequence->{'width'};
        $Str .= $title.' 'x($width-$base).$padding;
    }
    $Str .= "total (of shown problems)</pre>\n";
    $Str .= "<pre>";
    #
    # Check for suppression of output
    if ($show eq 'final table') {
        $Str = '';
    }
    $r->print($Str);
    $r->rflush();
    return;
}

sub html_outputstudent {
    my ($r,$student) = @_;
    my $Str = '';
    #
    if($count++ % 5 == 0 && $count > 0) {
        $r->print("</pre><pre>");
    }
    # First, the @StudentData fields need to be listed
    my @to_show = &get_student_fields_to_show();
    foreach my $field (@to_show) {
        my $title=$student->{$field};
        my $base = length($title);
        my $width=$Apache::lonstatistics::StudentData{$field}->{'width'};
        $Str .= $title.' 'x($width-$base).$padding;
    }
    # Get ALL the students data
    my %StudentsData;
    my @tmp = &Apache::loncoursedata::get_current_state
        ($student->{'username'},$student->{'domain'},undef,
         $ENV{'request.course.id'});
    if ((scalar @tmp > 0) && ($tmp[0] !~ /^error:/)) {
        %StudentsData = @tmp;
    }
    if (scalar(@tmp) < 1) {
        $nodata_count++;
        return if ($show eq 'final table');
        $Str .= '<font color="blue">No Course Data</font>'."\n";
        $r->print($Str);
        $r->rflush();
        return;
    }
    #
    # By sequence build up the data
    my $studentstats;
    my $PerformanceStr = '';
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my ($performance,$score,$seq_max) =
            &StudentPerformanceOnSequence($student,\%StudentsData,
                                          $seq,$show_links);
        my $ratio = $score.'/'.$seq_max;
        #
        if ($show eq 'totals') {
            $performance = ' 'x(length($seq_max)-length($score)).$ratio;
            $performance .= ' 'x($seq->{'width'}-length($performance));
        } elsif ($show eq 'scores') {
            $performance = $score;
            $performance .= ' 'x($seq->{'width'}-length($performance));
        } else {
            # Pad with extra spaces
            $performance .= ' 'x($seq->{'width'}-$seq_max-
                                 length($ratio)
                                 ).$ratio;
        }
        #
        $Str .= $performance.$padding;
        #
        $studentstats->{$seq->{'symb'}}->{'score'}= $score;
        $studentstats->{$seq->{'symb'}}->{'max'}  = $seq_max;
    }
    #
    # Total it up and store the statistics info.
    my ($score,$max) = (0,0);
    while (my ($symb,$seq_stats) = each (%{$studentstats})) {
        $Statistics->{$symb}->{'score'} += $seq_stats->{'score'};
        $Statistics->{$symb}->{'max'}   += $seq_stats->{'max'};
        $score += $seq_stats->{'score'};
        $max   += $seq_stats->{'max'};
    }
    $Str .= ' '.' 'x(length($max)-length($score)).$score.'/'.$max;
    $Str .= " \n";
    #
    # Check for suppressed output and update the progress window if so...
    if ($show eq 'final table') {
        $Str = '';
        &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,
                                                 'last student');
    }
    #
    $r->print($Str);
    #
    $r->rflush();
    return;
}    

sub html_finish {
    my ($r) = @_;
    #
    # Check for suppressed output and close the progress window if so
    if ($show eq 'final table') {
        &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    } else {
        $r->print("</pre>\n"); 
    }
    $r->print(&StudentAverageTotal());
    $r->rflush();
    return;
}

sub StudentAverageTotal {
    my $Str = "<h3>Summary Tables</h3>\n";
    my $num_students = scalar(@Apache::lonstatistics::Students);
    my $total_ave = 0;
    my $total_max = 0;
    $Str .= '<table border=2 cellspacing="1">'."\n";
    $Str .= "<tr><th>Title</th><th>Average</th><th>Maximum</th></tr>\n";
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my $ave;
        if ($num_students > $nodata_count) {
            $ave = int(100*($Statistics->{$seq->{'symb'}}->{'score'}/
                            ($num_students-$nodata_count)))/100;
        } else {
            $ave = 0;
        }
        $total_ave += $ave;
        my $max = $seq->{'num_assess_parts'};
        $total_max += $max;
        if ($ave == 0) {
            $ave = "0.00";
        }
        $ave .= '&nbsp;';
        $max .= '&nbsp;&nbsp;&nbsp;';
        $Str .= '<tr><td>'.$seq->{'title'}.'</td>'.
            '<td align="right">'.$ave.'</td>'.
                '<td align="right">'.$max.'</td></tr>'."\n";
    }
    $total_ave = int(100*$total_ave)/100; # only two digit
    $Str .= "</table>\n";
    $Str .= '<table border=2 cellspacing="1">'."\n";
    $Str .= '<tr><th>Number of Students</th><th>Average</th>'.
        "<th>Maximum</th></tr>\n";
    $Str .= '<tr><td>'.($num_students-$nodata_count).'</td>'.
        '<td>'.$total_ave.'</td><td>'.$total_max.'</td>';
    $Str .= "</table>\n";
    return $Str;
}

}

#######################################################
#######################################################

=pod

=head2 Multi-Sheet EXCEL subroutines

=item &multi_sheet_excel_initialize($r)

=item &multi_sheet_excel_outputstudent($r,$student)

=item &multi_sheet_excel_finish($r)

=cut

#######################################################
#######################################################
{

sub multi_sheet_excel_initialize {
    my ($r)=@_;
    $r->print("<h1>Not yet implemented</h1>");
    # 
    # Estimate the size of the file.  We would like to have < 5 megs of data.
    my $max_size = 5000000;
    my $num_students  = scalar(@Apache::lonstatistics::Students);
    my $num_sequences = 0;
    my $num_data_per_part  = 2; # 'status' and 'numtries'
    my $fields_per_student = scalar(&get_student_fields_to_show());
    my $bytes_per_field    = 20; # Back of the envelope calculation
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        $num_sequences++ if ($seq->{'num_assess'} > 0);
        $fields_per_student += $num_data_per_part * $seq->{'num_assess_parts'};
    }
    my $size_estimate = $fields_per_student*$num_students*$bytes_per_field;
    #
    # Compute number of workbooks
    my $num_workbooks = 1;
    if ($size_estimate > $max_size) { # try to stay under 5 megs
        $num_workbooks += int($size_estimate / $max_size);
    }
    if ($show eq 'by section') {
        if (@Apache::lonstatistics::SelectedSections > 1 && 
            $Apache::lonstatistics::SelectedSections[0] ne 'all') {
            $num_workbooks = scalar(@Apache::lonstatistics::SelectedSections);
        } else {
            # @Apache::lonstatistics::Sections contains 'all' as well.
            $num_workbooks = scalar(@Apache::lonstatistics::Sections) - 1;
        }
    }
    
    $r->print("Maximum allowed size: ".$max_size." bytes<br />");
    $r->print("Number of students: ".$num_students."<br />");
    $r->print("Number of fields per student: ".$fields_per_student."<br />");
    $r->print("Total number of fields: ".($fields_per_student*$num_students).
              "<br />");
    $r->print("Bytes per field: ".$bytes_per_field." (estimated)"."<br />");
    $r->print("Estimated size: ".$size_estimate." bytes<br />");
    $r->print("Number of workbooks: ".$num_workbooks."<br />");
    $r->rflush();
    return;
}

sub multi_sheet_excel_outputstudent {
    my ($r,$student) = @_;
}

sub multi_sheet_excel_finish {
    my ($r) = @_;
}

}
#######################################################
#######################################################

=pod

=head2 EXCEL subroutines

=item &excel_initialize($r)

=item &excel_outputstudent($r,$student)

=item &excel_finish($r)

=cut

#######################################################
#######################################################
{

my $excel_sheet;
my $excel_workbook;

my $filename;
my $rows_output;
my $cols_output;

my %prog_state; # progress window state

sub excel_initialize {
    my ($r) = @_;
    #
    $filename = '/prtspool/'.
        $ENV{'user.name'}.'_'.$ENV{'user.domain'}.'_'.
            time.'_'.rand(1000000000).'.xls';
    #
    $excel_workbook = undef;
    $excel_sheet = undef;
    #
    $rows_output = 0;
    $cols_output = 0;
    #
    # Create sheet
    $excel_workbook = Spreadsheet::WriteExcel->new('/home/httpd'.$filename);
    #
    # Check for errors
    if (! defined($excel_workbook)) {
        $r->log_error("Error creating excel spreadsheet $filename: $!");
        $r->print("Problems creating new Excel file.  ".
                  "This error has been logged.  ".
                  "Please alert your LON-CAPA administrator");
        return ;
    }
    #
    # The excel spreadsheet stores temporary data in files, then put them
    # together.  If needed we should be able to disable this (memory only).
    # The temporary directory must be specified before calling 'addworksheet'.
    # File::Temp is used to determine the temporary directory.
    $excel_workbook->set_tempdir($Apache::lonnet::tmpdir);
    #
    # Add a worksheet
    my $sheetname = $ENV{'course.'.$ENV{'request.course.id'}.'.description'};
    if (length($sheetname) > 31) {
        $sheetname = substr($sheetname,0,31);
    }
    $excel_sheet = $excel_workbook->addworksheet($sheetname);
    #
    # Put the course description in the header
    $excel_sheet->write($rows_output,$cols_output++,
                   $ENV{'course.'.$ENV{'request.course.id'}.'.description'});
    $cols_output += 3;
    #
    # Put a description of the sections listed
    my $sectionstring = '';
    my @Sections = @Apache::lonstatistics::SelectedSections;
    if (scalar(@Sections) > 1) {
        if (scalar(@Sections) > 2) {
            my $last = pop(@Sections);
            $sectionstring = "Sections ".join(', ',@Sections).', and '.$last;
        } else {
            $sectionstring = "Sections ".join(' and ',@Sections);
        }
    } else {
        if ($Sections[0] eq 'all') {
            $sectionstring = "All sections";
        } else {
            $sectionstring = "Section ".$Sections[0];
        }
    }
    $excel_sheet->write($rows_output,$cols_output++,$sectionstring);
    $cols_output += scalar(@Sections);
    #
    # Put the date in there too
    $excel_sheet->write($rows_output,$cols_output++,
                        'Compiled on '.localtime(time));
    #
    $rows_output++;
    #
    # Add the student headers
    $cols_output = 0;
    foreach my $field (&get_student_fields_to_show()) {
        $excel_sheet->write($rows_output,$cols_output++,$field);
    }
    #
    # Add the Sequence Headers
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        $excel_sheet->write($rows_output,$cols_output,$seq->{'title'});
        if ($show eq 'totals') {
            $excel_sheet->write($rows_output+1,$cols_output,'score');
            $excel_sheet->write($rows_output+1,$cols_output+1,'maximum');
            $cols_output += 2;
        } else {
            $cols_output++;
        }
    }
    #
    # Bookkeeping
    if ($show eq 'totals') {
        $rows_output += 2;
    } else {
        $rows_output += 1;
    }
    #
    # Output a row for MAX
    if ($show ne 'totals') {
        $cols_output = 0;
        foreach my $field (&get_student_fields_to_show()) {
            if ($field eq 'username' || $field eq 'fullname' || 
                $field eq 'id') {
                $excel_sheet->write($rows_output,$cols_output++,'Maximum');
            } else {
                $excel_sheet->write($rows_output,$cols_output++,'');
            }
        }
        #
        # Add the Sequence Headers
        foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
            $excel_sheet->write($rows_output,$cols_output++,
                                $seq->{'num_assess_parts'});
        }
        $rows_output++;
    }
    #
    # Let the user know what we are doing
    my $studentcount = scalar(@Apache::lonstatistics::Students); 
    $r->print("<h1>Compiling Excel spreadsheet for ".
              $studentcount.' student');
    $r->print('s') if ($studentcount > 1);
    $r->print("</h1>\n");
    $r->rflush();
    #
    # Initialize progress window
    %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,'Excel File Compilation Status',
         'Excel File Compilation Progress', $studentcount);
    #
    return;
}

sub excel_outputstudent {
    my ($r,$student) = @_;
    return if (! defined($excel_sheet));
    $cols_output=0;
    #
    # Write out student data
    my @to_show = &get_student_fields_to_show();
    foreach my $field (@to_show) {
        $excel_sheet->write($rows_output,$cols_output++,$student->{$field});
    }
    #
    # Get student assessment data
    my %StudentsData;
    my @tmp = &Apache::loncoursedata::get_current_state($student->{'username'},
                                                        $student->{'domain'},
                                                        undef,
                                                   $ENV{'request.course.id'});
    if ((scalar @tmp > 0) && ($tmp[0] !~ /^error:/)) {
        %StudentsData = @tmp;
    }
    #
    # Write out sequence scores and totals data
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my ($performance,$score,$seq_max) =
            &StudentPerformanceOnSequence($student,\%StudentsData,
                                          $seq,'no');
        if ($show eq 'totals' || $show eq 'scores') {
            $excel_sheet->write($rows_output,$cols_output++,$score);
        }
        if ($show eq 'totals') {
            $excel_sheet->write($rows_output,$cols_output++,$seq_max);
        }
    }
    #
    # Bookkeeping
    $rows_output++; 
    $cols_output=0;
    #
    # Update the progress window
    &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,'last student');
    return;
}

sub excel_finish {
    my ($r) = @_;
    return if (! defined($excel_sheet));
    #
    # Write the excel file
    $excel_workbook->close();
    my $c = $r->connection();
    #
    return if($c->aborted());
    #
    # Close the progress window
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    #
    # Tell the user where to get their excel file
    $r->print('<br />'.
              '<a href="'.$filename.'">Your Excel spreadsheet.</a>'."\n");
    $r->rflush();
    return;
}

}
#######################################################
#######################################################

=pod

=head2 CSV output routines

=item &csv_initialize($r)

=item &csv_outputstudent($r,$student)

=item &csv_finish($r)

=cut

#######################################################
#######################################################
{

my $outputfile;
my $filename;

my %prog_state; # progress window state

sub csv_initialize{
    my ($r) = @_;
    # 
    # Clean up
    $filename = undef;
    $outputfile = undef;
    undef(%prog_state);
    #
    # Open a file
    $filename = '/prtspool/'.
        $ENV{'user.name'}.'_'.$ENV{'user.domain'}.'_'.
            time.'_'.rand(1000000000).'.csv';
    unless ($outputfile = Apache::File->new('>/home/httpd'.$filename)) {
        $r->log_error("Couldn't open $filename for output $!");
        $r->print("Problems occured in writing the csv file.  ".
                  "This error has been logged.  ".
                  "Please alert your LON-CAPA administrator.");
        $outputfile = undef;
    }
    #
    # Datestamp
    my $description = $ENV{'course.'.$ENV{'request.course.id'}.'.description'};
    print $outputfile '"'.&Apache::loncommon::csv_translate($description).'",'.
        '"'.&Apache::loncommon::csv_translate(scalar(localtime(time))).'"'.
            "\n";

    #
    # Print out the headings
    my $Str = '';
    my $Str2 = undef;
    foreach my $field (&get_student_fields_to_show()) {
        if ($show eq 'scores') {
            $Str .= '"'.&Apache::loncommon::csv_translate($field).'",';
        } elsif ($show eq 'totals') {
            $Str .= '"",'; # first row empty on the student fields
            $Str2 .= '"'.&Apache::loncommon::csv_translate($field).'",';
        } elsif ($show eq 'all') {
            $Str .= '"'.&Apache::loncommon::csv_translate($field).'",';
        }
    }
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        if ($show eq 'scores') {
            $Str .= '"'.&Apache::loncommon::csv_translate($seq->{'title'}).
                '",';
        } elsif ($show eq 'totals') {
            $Str  .= '"'.&Apache::loncommon::csv_translate($seq->{'title'}).
                '","",';
            $Str2 .= '"score","total possible",';
        } elsif ($show eq 'all') {
            $Str  .= '"'.&Apache::loncommon::csv_translate($seq->{'title'}).
                '",';
            $Str .= '"",'x($seq->{'num_assess_parts'}-1);
            $Str .= '"score","total possible",';
        }
    }
    chop($Str);
    $Str .= "\n";
    print $outputfile $Str;
    if (defined($Str2)) {
        chop($Str2);
        $Str2 .= "\n";
        print $outputfile $Str2;
    }
    #
    # Initialize progress window
    my $studentcount = scalar(@Apache::lonstatistics::Students);
    %prog_state=&Apache::lonhtmlcommon::Create_PrgWin
        ($r,'CSV File Compilation Status',
         'CSV File Compilation Progress', $studentcount);
    return;
}

sub csv_outputstudent {
    my ($r,$student) = @_;
    return if (! defined($outputfile));
    my $Str = '';
    #
    # Output student fields
    my @to_show = &get_student_fields_to_show();
    foreach my $field (@to_show) {
        $Str .= '"'.&Apache::loncommon::csv_translate($student->{$field}).'",';
    }
    #
    # Get student assessment data
    my %StudentsData;
    my @tmp = &Apache::loncoursedata::get_current_state($student->{'username'},
                                                        $student->{'domain'},
                                                        undef,
                                                   $ENV{'request.course.id'});
    if ((scalar @tmp > 0) && ($tmp[0] !~ /^error:/)) {
        %StudentsData = @tmp;
    }
    #
    # Output performance data
    foreach my $seq (&Apache::lonstatistics::Sequences_with_Assess()) {
        my ($performance,$score,$seq_max) =
            &StudentPerformanceOnSequence($student,\%StudentsData,
                                          $seq,'no');
        if ($show eq 'scores') {
            $Str .= '"'.$score.'",';
        } elsif ($show eq 'totals') {
            $Str  .= '"'.$score.'","'.$seq_max.'",';
        } elsif ($show eq 'all') {
            $Str .= '"'.join('","',(split(//,$performance),$score,$seq_max)).
                '",';
        }
    }
    chop($Str);
    $Str .= "\n";
    print $outputfile $Str;
    #
    # Update the progress window
    &Apache::lonhtmlcommon::Increment_PrgWin($r,\%prog_state,'last student');
    return;
}

sub csv_finish {
    my ($r) = @_;
    return if (! defined($outputfile));
    close($outputfile);
    #
    my $c = $r->connection();
    return if ($c->aborted());
    #
    # Close the progress window
    &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
    #
    # Tell the user where to get their csv file
    $r->print('<br />'.
              '<a href="'.$filename.'">Your csv file.</a>'."\n");
    $r->rflush();
    return;
    
}

}

#######################################################
#######################################################

=pod

=item &StudentPerformanceOnSequence()

Inputs:

=over 4

=item $student

=item $studentdata Hash ref to all student data

=item $seq Hash ref, the sequence we are working on

=item $links if defined we will output links to each resource.

=back

=cut

#######################################################
#######################################################
sub StudentPerformanceOnSequence {
    my ($student,$studentdata,$seq,$links) = @_;
    $links = 'no' if (! defined($links));
    my $Str = '';
    my ($sum,$max) = (0,0);
    foreach my $resource (@{$seq->{'contents'}}) {
        next if ($resource->{'type'} ne 'assessment');
        my $resource_data = $studentdata->{$resource->{'symb'}};
        my $value = '';
        foreach my $partnum (@{$resource->{'parts'}}) {
            $max++;
            my $symbol = ' '; # default to space
            #
            if (exists($resource_data->{'resource.'.$partnum.'.solved'})) {
                my $status = $resource_data->{'resource.'.$partnum.'.solved'};
                if ($status eq 'correct_by_override') {
                    $symbol = '+';
                    $sum++;
                } elsif ($status eq 'incorrect_by_override') {
                    $symbol = '-';
                } elsif ($status eq 'ungraded_attempted') {
                    $symbol = '#';
                } elsif ($status eq 'incorrect_attempted')  {
                    $symbol = '.';
                } elsif ($status eq 'excused') {
                    $symbol = 'x';
                    $max--;
                } elsif ($status eq 'correct_by_student' &&
                    exists($resource_data->{'resource.'.$partnum.'.tries'})){
                    my $num = $resource_data->{'resource.'.$partnum.'.tries'};
                    if ($num > 9) {
                        $symbol = '*';
                    } elsif ($num > 0) {
                        $symbol = $num;
                    } else {
                        $symbol = ' ';
                    }
                    $sum++;
                } elsif (exists($resource_data->{'resource.'.
                                                     $partnum.'.tries'})){
                    $symbol = '.';
                } else {
                    $symbol = ' ';
                }
            } else {
                # Unsolved.  Did they try?
                if (exists($resource_data->{'resource.'.$partnum.'.tries'})){
                    $symbol = '.';
                } else {
                    $symbol = ' ';
                }
            }
            #
            if ( ($links eq 'yes' && $symbol ne ' ') ||
                 ($links eq 'all')) {
                $symbol = '<a href="/adm/grades'.
                    '?symb='.&Apache::lonnet::escape($resource->{'symb'}).
                        '&student='.$student->{'username'}.
                            '&domain='.$student->{'domain'}.
                                '&command=submission">'.$symbol.'</a>';
            }
            $value .= $symbol;
        }
        $Str .= $value;
    }
    return ($Str,$sum,$max);
}

#######################################################
#######################################################

=pod

=item &CreateLegend()

This function returns a formatted string containing the legend for the
chart.  The legend describes the symbols used to represent grades for
problems.

=cut

#######################################################
#######################################################
sub CreateLegend {
    my $Str = "<p><pre>".
              "   1  correct by student in 1 try\n".
              "   7  correct by student in 7 tries\n".
              "   *  correct by student in more than 9 tries\n".
	      "   +  correct by hand grading or override\n".
              "   -  incorrect by override\n".
	      "   .  incorrect attempted\n".
	      "   #  ungraded attempted\n".
              "      not attempted (blank field)\n".
	      "   x  excused".
              "</pre><p>";
    return $Str;
}

#######################################################
#######################################################

=pod 

=back

=cut

#######################################################
#######################################################

1;

__END__
